class AttributeToScopePatternConfigurationObjects:
    def __init__(
        self,
        attributed_type_name: str,
        attributed_type_ea_guid: str,
        attribute_name: str,
        attribute_ea_guid: str,
        scoping_type: str,
        scoping_type_name: str,
    ):
        self.attributed_type_name = (
            attributed_type_name
        )

        self.attributed_type_ea_guid = (
            attributed_type_ea_guid
        )

        self.attribute_name = (
            attribute_name
        )

        self.attribute_ea_guid = (
            attribute_ea_guid
        )

        self.scoping_type = scoping_type

        self.scoping_type_name = (
            scoping_type_name
        )

    def __enter__(self):
        return self

    def __exit__(
        self,
        exception_type,
        exception_value,
        traceback,
    ):
        pass



# TODO: this should not be needed,need a pandas concat function
import pandas as pd
from bclearer_core.constants.standard_constants import (
    DEFAULT_NULL_VALUE,
)
from bclearer_core.nf.types.nf_column_types import (
    NfColumnTypes,
)
from bclearer_core.substages.operations.common.ea_guid_from_nf_uuid_creator import (
    create_ea_guid_from_nf_uuid,
)
from bclearer_interop_services.ea_interop_service.general.nf_ea.com.common_knowledge.collection_types.nf_ea_com_collection_types import (
    NfEaComCollectionTypes,
)
from bclearer_interop_services.ea_interop_service.general.nf_ea.com.common_knowledge.column_types.nf_ea_com_column_types import (
    NfEaComColumnTypes,
)
from bclearer_interop_services.ea_interop_service.general.nf_ea.com.nf_ea_com_universes import (
    NfEaComUniverses,
)
from bclearer_interop_services.ea_interop_service.nf_ea_common.common_knowledge.column_types.ea_t.ea_t_package_column_types import (
    EaTPackageColumnTypes,
)
from bclearer_interop_services.ea_interop_service.nf_ea_common.common_knowledge.ea_element_types import (
    EaElementTypes,
)
from bclearer_orchestration_services.identification_services.uuid_service.uuid_helpers.uuid_factory import (
    create_new_uuid,
)


def create_root_package(
    nf_ea_com_universe: NfEaComUniverses,
    package_name: str,
) -> str:
    package_collection = nf_ea_com_universe.nf_ea_com_registry.dictionary_of_collections[
        NfEaComCollectionTypes.EA_PACKAGES
    ]

    package_nf_uuid = create_new_uuid()

    package_ea_guid = (
        create_ea_guid_from_nf_uuid(
            nf_uuid=package_nf_uuid,
        )
    )

    package_ea_package_path = (
        package_name
    )

    ea_package_dictionary = {
        EaTPackageColumnTypes.T_PACKAGE_EA_GUIDS.nf_column_name: package_ea_guid,
        "ea_package_path": package_ea_package_path,
        "parent_package_ea_guid": DEFAULT_NULL_VALUE,
        "parent_package_name": DEFAULT_NULL_VALUE,
        NfColumnTypes.NF_UUIDS.column_name: package_nf_uuid,
        NfEaComColumnTypes.PACKAGES_CONTAINED_EA_PACKAGES.column_name: [],
        NfEaComColumnTypes.ELEMENTS_EA_OBJECT_TYPE.column_name: EaElementTypes.PACKAGE.type_name,
        NfEaComColumnTypes.ELEMENTS_SUPPLIER_PLACE1_END_CONNECTORS.column_name: [],
        NfEaComColumnTypes.ELEMENTS_CLIENT_PLACE2_END_CONNECTORS.column_name: [],
        NfEaComColumnTypes.ELEMENTS_CONTAINED_EA_DIAGRAMS.column_name: [],
        NfEaComColumnTypes.ELEMENTS_CONTAINED_EA_CLASSIFIERS.column_name: [],
        NfEaComColumnTypes.PACKAGEABLE_OBJECTS_PARENT_EA_ELEMENT.column_name: DEFAULT_NULL_VALUE,
        NfEaComColumnTypes.STEREOTYPEABLE_OBJECTS_EA_OBJECT_STEREOTYPES.column_name: [],
        NfEaComColumnTypes.EXPLICIT_OBJECTS_EA_OBJECT_NAME.column_name: package_name,
        NfEaComColumnTypes.EXPLICIT_OBJECTS_EA_GUID.column_name: package_ea_guid,
    }

    new_row_df = pd.DataFrame(
        [ea_package_dictionary]
    )

    updated_package_collection = (
        pd.concat(
            [
                package_collection,
                new_row_df,
            ],
            ignore_index=True,
        )
    )

    nf_ea_com_universe.nf_ea_com_registry.dictionary_of_collections[
        NfEaComCollectionTypes.EA_PACKAGES
    ] = updated_package_collection

    return package_nf_uuid



from bclearer_core.bie.infrastructure.types.bie_common_column_names import (
    BieCommonColumnNames,
)
from bclearer_core.constants.standard_constants import (
    HR_DATE_TIME_FORMAT,
)
from bclearer_core.infrastructure.session.os_user_session.enums.os_user_profile_session_object_column_names import (
    OsUserProfileSessionObjectColumnNames,
)
from bclearer_core.infrastructure.session.os_user_session.os_user_profile_session_objects import (
    OsUserProfileSessionObjects,
)
from bclearer_interop_services.b_dictionary_service.table_as_dictionary_service.table_as_dictionary_to_dataframe_converter import (
    convert_table_as_dictionary_to_dataframe,
)
from bclearer_orchestration_services.identification_services.b_identity_ecosystem.common_knowledge.bie_column_names import (
    BieColumnNames,
)
from pandas import DataFrame


def create_os_user_profile_session_objects_table() -> (
    DataFrame
):
    table_as_dictionary = (
        __get_os_user_profile_session_object_table_as_dictionary()
    )

    table_as_dataframe = convert_table_as_dictionary_to_dataframe(
        table_as_dictionary=table_as_dictionary
    )

    return table_as_dataframe.astype(
        str
    )


def __get_os_user_profile_session_object_table_as_dictionary() -> (
    dict
):
    os_user_profile_session_object = (
        OsUserProfileSessionObjects()
    )

    session_start_date_time = os_user_profile_session_object.session_start_date_time.strftime(
        format=HR_DATE_TIME_FORMAT
    )

    os_user_profile_session_object_row = {
        BieColumnNames.BIE_IDS.b_enum_item_name: os_user_profile_session_object.bie_id,
        BieCommonColumnNames.BASE_HR_NAMES.b_enum_item_name: os_user_profile_session_object.base_hr_name,
        BieCommonColumnNames.BIE_INFRASTRUCTURE_TYPE_IDS.b_enum_item_name: os_user_profile_session_object.bie_type.item_bie_identity,
        OsUserProfileSessionObjectColumnNames.WINDOWS_SESSION_IDS.b_enum_item_name: os_user_profile_session_object.windows_session_id,
        OsUserProfileSessionObjectColumnNames.SESSION_START_DATE_TIMES.b_enum_item_name: session_start_date_time,
        OsUserProfileSessionObjectColumnNames.OWNING_OS_USER_PROFILE_BIE_IDS.b_enum_item_name: os_user_profile_session_object.owning_os_user_profile.bie_id,
        OsUserProfileSessionObjectColumnNames.OWNING_OPERATING_SYSTEM_BIE_IDS.b_enum_item_name: os_user_profile_session_object.owning_operating_system_object.bie_id,
    }

    table_as_dictionary = {
        0: os_user_profile_session_object_row
    }

    return table_as_dictionary



import socket

from bclearer_core.infrastructure.session.common.session_identity_vectors import (
    OperatingSystemObjectIdentityVector,
    OperatingSystemObjectIdentityVectorPlaces,
)
from bclearer_core.infrastructure.session.common.session_objects import (
    SessionObjects,
)
from bclearer_core.infrastructure.session.operating_system.helpers.bie_operating_system_type_getter import (
    get_bie_operating_system_type,
)
from bclearer_core.infrastructure.session.operating_system.helpers.machine_sid_getter import (
    get_machine_sid,
)
from bclearer_orchestration_services.identification_services.b_identity_ecosystem.bie_id_creation_module.bie_id_creation_facade import (
    BieIdCreationFacade,
)
from bclearer_orchestration_services.identification_services.b_identity_ecosystem.objects.bie_ids import (
    BieIds,
)


class OperatingSystemObjects(
    SessionObjects
):
    _instance = None

    def __new__(cls):
        if not cls._instance:
            cls._instance = super(
                OperatingSystemObjects,
                cls,
            ).__new__(cls)

        return cls._instance

    def __init__(self):
        # TODO: get: Computer Name, Machine SID
        # NOTE: Install date deprecated
        # TODO: get bie operating system type (enum) - use standard getter
        # TODO: check it is machine name
        self.computer_name = (
            socket.gethostname()
        )

        self.machine_sid = (
            get_machine_sid()
        )

        self.bie_operating_system_type = (
            get_bie_operating_system_type()
        )

        # TODO: The following attributes have still not been agreed
        # TODO: If this changes after updates we may not want it to identify this object - discuss

        # NOTE: Install date deprecated
        # self.install_date = \
        #     get_os_install_date()

        # NOTE: version deprecated
        # self.version = \
        #     platform.version()

        # NOTE: release deprecated
        # self.release = \
        #     platform.release()

        # NOTE: operating_system_name deprecated
        # self.name = \
        #     self.__get_operating_system_name()

        base_bie_id = (
            self.__create_bie_id()
        )

        super().__init__(
            bie_id=base_bie_id,
            base_hr_name=self.computer_name,
            bie_infrastructure_type=self.bie_operating_system_type,
        )

    def __create_bie_id(self) -> BieIds:
        identity_vector = OperatingSystemObjectIdentityVector(
            places=OperatingSystemObjectIdentityVectorPlaces(
                operating_system_type_bie_id=self.bie_operating_system_type.item_bie_identity,
                machine_sid=self.machine_sid,
            )
        )

        bie_id = BieIdCreationFacade.create_bie_id_from_identity_vector(
            identity_vector=identity_vector,
        )

        return bie_id



"""Template for b_unit class using object passing."""

B_UNIT_WITH_OBJECT_PASSING_TEMPLATE = """from bclearer_core.objects.b_units import (
    BUnitsWithObjectPassingAndReturning,
)


class {class_name}BUnits(BUnitsWithObjectPassingAndReturning):
    def __init__(self, input_object=None):
        super().__init__(
            input_object=input_object
        )

    def _b_unit_process_function(
        self,
    ) -> None:
        pass
"""



from bclearer_core.infrastructure.session.common.enums.operating_system_kind import (
    OperatingSystemKind,
)
from bclearer_core.infrastructure.session.common.platform_type_getter import (
    get_operating_system_kind,
)
from bclearer_core.infrastructure.session.user.helpers.unix.user_identifier import (
    get_unix_user_sid,
)


def get_user_sid():
    operating_system_kind = (
        get_operating_system_kind()
    )

    if (
        operating_system_kind
        is OperatingSystemKind.WINDOWS
    ):
        from bclearer_core.infrastructure.session.user.helpers.windows.security import (
            get_windows_user_sid,
        )

        return get_windows_user_sid()

    if operating_system_kind in {
        OperatingSystemKind.LINUX,
        OperatingSystemKind.MACOS,
    }:
        return get_unix_user_sid()

    return get_unix_user_sid()



from bclearer_core.common_knowledge.bclearer_constants import (
    NAME_INSTANCE_ATTRIBUTE_NAME,
)
from bclearer_core.common_knowledge.bclearer_matched_ea_objects import (
    BclearerMatchedEaObjects,
)
from bclearer_core.nf.types.nf_column_types import (
    NfColumnTypes,
)
from bclearer_core.substages.operations.b_evolve.convention_shift_operations.convention_shifters.common.new_subpackage_creator import (
    create_new_subpackage_if_not_exist,
)
from bclearer_core.substages.operations.common.attribute_adder import (
    add_new_attribute_to_dictionary,
)
from bclearer_core.substages.operations.common.class_adder import (
    add_new_class_to_dictionary,
)
from bclearer_core.substages.operations.common.connector_adder import (
    add_new_connector_to_dictionary,
)
from bclearer_core.substages.operations.common.new_ea_objects_dictionary_creator import (
    create_new_ea_objects_dictionary,
)
from bclearer_core.substages.operations.common.nf_ea_com_universe_updater import (
    update_nf_ea_com_universe_with_dictionary,
)
from bclearer_core.substages.operations.common.nf_uuid_from_ea_guid_from_collection_getter import (
    get_nf_uuid_from_ea_guid_from_collection,
)
from bclearer_core.substages.operations.common.stereotype_adder import (
    add_new_stereotype_usage_to_dictionary,
)
from bclearer_interop_services.ea_interop_service.general.nf_ea.com.common_knowledge.collection_types.nf_ea_com_collection_types import (
    NfEaComCollectionTypes,
)
from bclearer_interop_services.ea_interop_service.general.nf_ea.com.common_knowledge.column_types.nf_ea_com_column_types import (
    NfEaComColumnTypes,
)
from bclearer_interop_services.ea_interop_service.general.nf_ea.com.nf_ea_com_universes import (
    NfEaComUniverses,
)
from bclearer_interop_services.ea_interop_service.nf_ea_common.common_knowledge.ea_connector_types import (
    EaConnectorTypes,
)
from bclearer_interop_services.tuple_service.tuple_attribute_value_getter import (
    get_tuple_attribute_value_if_required,
)
from bclearer_orchestration_services.reporting_service.reporters.log_with_datetime import (
    log_message,
)
from pandas import DataFrame


def shift_convention_attribute_names_to_name_objects(
    content_universe: NfEaComUniverses,
    list_of_configuration_objects: list,
    output_universe: NfEaComUniverses,
) -> NfEaComUniverses:
    log_message(
        message="CONVENTION SHIFT OPERATION: Extract attribute names to name objects - started",
    )

    naming_space_instances_ea_guids = [
        configuration_object.matched_naming_space_instance.ea_guid
        for configuration_object in list_of_configuration_objects
    ]

    output_universe.nf_ea_com_registry.dictionary_of_collections = (
        content_universe.nf_ea_com_registry.dictionary_of_collections.copy()
    )

    output_universe_ea_classifiers = (
        output_universe.nf_ea_com_registry.get_ea_classifiers()
    )

    output_universe_ea_attributes = (
        output_universe.nf_ea_com_registry.get_ea_attributes()
    )

    naming_space_instances_nf_uuids = output_universe_ea_classifiers[
        output_universe_ea_classifiers[
            NfEaComColumnTypes.EXPLICIT_OBJECTS_EA_GUID.column_name
        ].isin(
            naming_space_instances_ea_guids,
        )
    ][
        NfColumnTypes.NF_UUIDS.column_name
    ]

    output_universe_name_attributes = output_universe_ea_attributes[
        output_universe_ea_attributes[
            NfEaComColumnTypes.ELEMENT_COMPONENTS_CLASSIFYING_EA_CLASSIFIER.column_name
        ].isin(
            naming_space_instances_nf_uuids,
        )
    ]

    named_by_stereotype_nf_uuid = get_nf_uuid_from_ea_guid_from_collection(
        nf_ea_com_universe=content_universe,
        collection_type=NfEaComCollectionTypes.EA_STEREOTYPES,
        ea_guid=BclearerMatchedEaObjects.NAMED_BY_STEREOTYPE.ea_guid,
    )

    character_string_nf_uuid = get_nf_uuid_from_ea_guid_from_collection(
        nf_ea_com_universe=content_universe,
        collection_type=NfEaComCollectionTypes.EA_CLASSIFIERS,
        ea_guid=BclearerMatchedEaObjects.CHARACTER_STRINGS.ea_guid,
    )

    new_ea_objects_dictionary = (
        create_new_ea_objects_dictionary()
    )

    output_universe_ea_classifiers = (
        output_universe.nf_ea_com_registry.get_ea_classifiers()
    )

    output_universe_ea_packages = (
        output_universe.nf_ea_com_registry.get_ea_packages()
    )

    for (
        name_attribute_tuple
    ) in (
        output_universe_name_attributes.itertuples()
    ):
        __convert_name_attribute_into_name_object(
            output_universe=output_universe,
            output_universe_ea_classifiers=output_universe_ea_classifiers,
            output_universe_ea_packages=output_universe_ea_packages,
            name_attribute_tuple=name_attribute_tuple,
            new_ea_objects_dictionary=new_ea_objects_dictionary,
            named_by_stereotype_nf_uuid=named_by_stereotype_nf_uuid,
            character_string_nf_uuid=character_string_nf_uuid,
        )

    update_nf_ea_com_universe_with_dictionary(
        nf_ea_com_universe=output_universe,
        new_ea_objects_dictionary=new_ea_objects_dictionary,
    )

    log_message(
        message="CONVENTION SHIFT OPERATION: Extract attribute names to name objects - finished",
    )

    return output_universe


def __update_collection_with_dictionary(
    output_universe: NfEaComUniverses,
    dictionary: dict,
    collection_type: NfEaComCollectionTypes,
) -> None:
    new_collection = (
        DataFrame.from_dict(
            data=dictionary,
            orient="index",
        )
    )

    output_universe.nf_ea_com_registry.update(
        collection_type=collection_type,
        new_collection=new_collection,
    )


def __convert_name_attribute_into_name_object(
    output_universe: NfEaComUniverses,
    output_universe_ea_classifiers: DataFrame,
    output_universe_ea_packages: DataFrame,
    name_attribute_tuple: tuple,
    new_ea_objects_dictionary: dict,
    named_by_stereotype_nf_uuid: str,
    character_string_nf_uuid: str,
):
    attributed_object_nf_uuid = get_tuple_attribute_value_if_required(
        owning_tuple=name_attribute_tuple,
        attribute_name=NfEaComColumnTypes.ELEMENT_COMPONENTS_CONTAINING_EA_CLASSIFIER.column_name,
    )

    attributed_object_package_nf_uuid = output_universe_ea_classifiers[
        output_universe_ea_classifiers[
            NfColumnTypes.NF_UUIDS.column_name
        ]
        == attributed_object_nf_uuid
    ][
        NfEaComColumnTypes.PACKAGEABLE_OBJECTS_PARENT_EA_ELEMENT.column_name
    ].values[
        0
    ]

    name_type_nf_uuid = get_tuple_attribute_value_if_required(
        owning_tuple=name_attribute_tuple,
        attribute_name=NfEaComColumnTypes.ELEMENT_COMPONENTS_CLASSIFYING_EA_CLASSIFIER.column_name,
    )

    name_attribute_name = get_tuple_attribute_value_if_required(
        owning_tuple=name_attribute_tuple,
        attribute_name=NfEaComColumnTypes.EXPLICIT_OBJECTS_EA_OBJECT_NAME.column_name,
    )

    name_attribute_type = get_tuple_attribute_value_if_required(
        owning_tuple=name_attribute_tuple,
        attribute_name=NfEaComColumnTypes.ELEMENT_COMPONENTS_TYPE.column_name,
    )

    name_object_name = (
        name_attribute_name
        + " "
        + name_attribute_type[:-1]
    )

    name_object_nf_uuid = __create_name_object(
        output_universe_ea_packages=output_universe_ea_packages,
        new_ea_packages_dictionary=new_ea_objects_dictionary[
            NfEaComCollectionTypes.EA_PACKAGES
        ],
        new_ea_classifiers_dictionary=new_ea_objects_dictionary[
            NfEaComCollectionTypes.EA_CLASSIFIERS
        ],
        package_nf_uuid=attributed_object_package_nf_uuid,
        name_object_name=name_object_name,
    )

    add_new_connector_to_dictionary(
        new_connector_dictionary=new_ea_objects_dictionary[
            NfEaComCollectionTypes.EA_CONNECTORS
        ],
        place_1_nf_uuid=name_object_nf_uuid,
        place_2_nf_uuid=name_type_nf_uuid,
        connector_type=EaConnectorTypes.DEPENDENCY,
    )

    name_to_attributed_object_association_nf_uuid = add_new_connector_to_dictionary(
        new_connector_dictionary=new_ea_objects_dictionary[
            NfEaComCollectionTypes.EA_CONNECTORS
        ],
        place_1_nf_uuid=name_object_nf_uuid,
        place_2_nf_uuid=attributed_object_nf_uuid,
        connector_type=EaConnectorTypes.ASSOCIATION,
    )

    add_new_stereotype_usage_to_dictionary(
        new_stereotype_usage_dictionary=new_ea_objects_dictionary[
            NfEaComCollectionTypes.STEREOTYPE_USAGE
        ],
        client_nf_uuid=name_to_attributed_object_association_nf_uuid,
        client_collection_type=NfEaComCollectionTypes.EA_CONNECTORS,
        stereotype_nf_uuid=named_by_stereotype_nf_uuid,
    )

    name_attribute_nf_uuid = get_tuple_attribute_value_if_required(
        owning_tuple=name_attribute_tuple,
        attribute_name=NfColumnTypes.NF_UUIDS.column_name,
    )

    __remove_attribute(
        nf_ea_com_universe=output_universe,
        attribute_nf_uuid=name_attribute_nf_uuid,
    )

    add_new_attribute_to_dictionary(
        new_ea_attributes_dictionary=new_ea_objects_dictionary[
            NfEaComCollectionTypes.EA_ATTRIBUTES
        ],
        name_object_nf_uuid=name_object_nf_uuid,
        attribute_value=name_attribute_name,
        attribute_name=NAME_INSTANCE_ATTRIBUTE_NAME,
        attribute_type_nf_uuid=character_string_nf_uuid,
    )


def __create_name_object(
    output_universe_ea_packages: DataFrame,
    new_ea_packages_dictionary: dict,
    new_ea_classifiers_dictionary: dict,
    package_nf_uuid: str,
    name_object_name: str,
) -> str:
    names_subpackage_nf_uuid = create_new_subpackage_if_not_exist(
        nf_ea_com_universe_ea_packages=output_universe_ea_packages,
        new_ea_packages_dictionary=new_ea_packages_dictionary,
        package_nf_uuid=package_nf_uuid,
        new_subpackage_name="Names",
    )

    name_object_nf_uuid = add_new_class_to_dictionary(
        new_classifier_dictionary=new_ea_classifiers_dictionary,
        package_nf_uuid=names_subpackage_nf_uuid,
        class_name=name_object_name,
    )

    return name_object_nf_uuid


def __remove_attribute(
    nf_ea_com_universe: NfEaComUniverses,
    attribute_nf_uuid: str,
):
    ea_attributes = (
        nf_ea_com_universe.nf_ea_com_registry.get_ea_attributes()
    )

    updated_ea_attributes = ea_attributes[
        ea_attributes[
            NfColumnTypes.NF_UUIDS.column_name
        ]
        != attribute_nf_uuid
    ]

    nf_ea_com_universe.nf_ea_com_registry.replace_collection(
        collection_type=NfEaComCollectionTypes.EA_ATTRIBUTES,
        collection=updated_ea_attributes,
    )



from bclearer_core.configurations.b_clearer_configurations.b_clearer_configurations import (
    BClearerConfigurations,
)


class BieConfigurations(
    BClearerConfigurations
):
    ENABLE_BIE_UNIVERSE_INSPECTION = (
        True
    )

    B_SEQUENCE_NAME_TRUNCATION_LEVEL = 0
    #
    # DEFAULT_DIGEST_SIZE = \
    #     8
